@@ -6,11 +6,199 @@
 
 #include <stdio.h>
 #include "sudoku.h"
-
 /*
  ******************************************************************************/
 int forcing_chain(t_sudoku *sudoku, int verbosity, int *fcl) {
 	
+	int size = sudoku->size;
+	int r, c, cc, row, col, cand, c0, c1, elim[NUM_ET], ret, eofc0, eofc1;
+	int minr, minc, mincand, minfcl, minrv, minstartr, minstartc;
+	int fcrow, fccol, fccand, fclen, fcrv, fcstartr, fcstartc;
+	int depth;
+	status minstat, fcstat;
+	char estr[512];
+	t_sudoku sudokuc0, sudokuc1;
+	
+	t_field diffgridc0[SIZE2][SIZE2];
+	t_field diffgridc1[SIZE2][SIZE2];
+	t_field intgrid[SIZE2][SIZE2];
+
+	extern int dfcs[];
+
+	// minfcl is minimum length of forcing chain, minrv is return value
+	// of this function
+	minfcl = size*size;
+	minrv = 0;
+	
+	// allowed techniques during forcing chain: .-dDbBtTqQ
+	depth = 1;
+	while ( (depth < (NUM_ET-1))
+			&& (       (dfcs[depth] == et_hs)
+					|| (dfcs[depth] == et_s)
+					|| (dfcs[depth] == et_hd)
+					|| (dfcs[depth] == et_d)
+					|| (dfcs[depth] == et_l)
+					|| (dfcs[depth] == et_b)
+					|| (dfcs[depth] == et_ht)
+					|| (dfcs[depth] == et_t)
+					|| (dfcs[depth] == et_hq)
+					|| (dfcs[depth] == et_q) ) ) {
+		depth++;
+	}
+	depth--;
+	
+	
+	// loop over all cells with only two candidates
+	for (r=0; r<size; r++) {
+		for (c=0; c<size; c++) {
+			if (sudoku->grid[r][c].ncand == 2) {
+				
+				// copy grid twice
+				copy_sudoku(*sudoku,&sudokuc0);
+				copy_sudoku(*sudoku,&sudokuc1);
+				
+				// put candidates into copied grids
+				cc = 0;
+				while ((cc < size) && (sudoku->grid[r][c].cand[cc] == 0)) cc++;
+				c0 = cc;
+				sudokuc0.grid[r][c].value = sudoku->grid[r][c].cand[cc];
+				sudokuc0.grid[r][c].stat = stat_guess;
+				remove_candidates(&sudokuc0,r,c);
+
+				cc++;
+				while ((cc < size) && (sudoku->grid[r][c].cand[cc] == 0)) cc++;
+				c1 = cc;
+				sudokuc1.grid[r][c].value = sudoku->grid[r][c].cand[cc];
+				sudokuc1.grid[r][c].stat = stat_guess;
+				remove_candidates(&sudokuc1,r,c);
+
+				// continue elimination until there is a corresponding cell value or
+				// until end of forcing chain
+				fclen = eofc0 = eofc1 = fcrv = 0;				
+				do {
+					// fill in hidden single or single
+					if ((eofc0 == 0) && (apply_solution_technique1(&sudokuc0,elim,depth,0,stdout) <= 0)) {
+						ret = check_sudoku(sudokuc0);
+						if (ret == 0) {
+							// end of sudoku: row r, col c, cand c0+1, end of loop
+							fcrow = r;
+							fccol = c;
+							fccand = c0+1;
+							fcstat = stat_guess; // have to include this cell for uniqueness check
+							fcrv = 1;
+							eofc0 = eofc1 = 1;
+						} else if (ret < 0) {
+							// contradiction: row r, col c, cand c1+1, end of loop
+							fcrow = r;
+							fccol = c;
+							fccand = c1+1;
+							fcstat = stat_fchain;
+							fcrv = 2;
+							eofc0 = eofc1 = 1;
+						} else {
+							// end of this part of forcing chain
+							eofc0 = 1;
+						}
+					}
+					if ((eofc1 == 0) && (apply_solution_technique1(&sudokuc1,elim,depth,0,stdout) <= 0)) {
+						ret = check_sudoku(sudokuc1);
+						if (ret == 0) {
+							// end of sudoku: row r, col c, cand c1+1, end of loop
+							fcrow = r;
+							fccol = c;
+							fccand = c1+1;
+							fcstat = stat_guess; // have to include this cell for uniqueness check
+							fcrv = 1;
+							eofc0 = eofc1 = 1;
+						} else if (ret < 0) {
+							// contradiction: row r, col c, cand c0+1, end of loop
+							fcrow = r;
+							fccol = c;
+							fccand = c0+1;
+							fcstat = stat_fchain;
+							fcrv = 2;
+							eofc0 = eofc1 = 1;
+						} else {
+							// end of forcing chain: check gridc0 and gridc1 for new common cells
+							eofc1 = 1;
+						}
+					}
+					// increment length of forcing chain
+					fclen++;
+
+					// now look for new common values in gridc0 and gridc1
+					if ((eofc0 == 0) || (eofc1 == 0)) {
+						diff_grid(sudoku->grid,sudokuc0.grid,diffgridc0,size);
+						diff_grid(sudoku->grid,sudokuc1.grid,diffgridc1,size);
+						intersect_grid_no_cands(diffgridc0,diffgridc1,intgrid,size);
+						intgrid[r][c].cand[c0] = 0;
+						intgrid[r][c].cand[c1] = 0;
+						intgrid[r][c].value = 0;
+						intgrid[r][c].stat = stat_free;
+						for (row=0; row<size; row++) {
+							for (col=0; col<size; col++) {
+								if ((cand = intgrid[row][col].value) != 0) {
+									// forcing chain successful, end of loop
+									fcrow = row;
+									fccol = col;
+									fccand = cand;
+									fcstat = stat_fchain;
+									fcstartr = r;
+									fcstartc = c;
+									fcrv = 3;
+									eofc0 = eofc1 = 1;
+								}
+							}
+						}
+					}
+				} while ((eofc0 == 0) || (eofc1 == 0));
+				
+				// if current forcing chain length is smaller than the ones before ...
+				if ((fcrv > 0) && (fclen < minfcl)) {
+					minr = fcrow;
+					minc = fccol;
+					minfcl = fclen;
+					mincand = fccand;
+					minstat = fcstat;
+					minstartr = fcstartr;
+					minstartc = fcstartc;
+					minrv = fcrv;
+				}
+			}
+			// look for next cell with 2 candidates			
+		}
+	}
+	
+	// if there is a forcing chain, put its result into grid
+	if (minfcl < size*size) {
+		sudoku->grid[minr][minc].value = mincand;
+		sudoku->grid[minr][minc].stat = minstat;
+		remove_candidates(sudoku,minr,minc);
+		*fcl = minfcl;
+		switch (minrv) {
+			case 1:
+				snprintf(estr,511,"e  r%dc%d, (%d) forcing chain of length %d to end",minr+1,minc+1,mincand,minfcl);
+				break;
+			case 2:
+				snprintf(estr,511,"f  r%dc%d, (%d) forcing chain of length %d to contradiction",minr+1,minc+1,mincand,minfcl);
+				break;
+			case 3:
+				snprintf(estr,511,"F  forcing chain of length %d starting at r%dc%d to common value (%d) at r%dc%d",minfcl,minstartr+1,minstartc+1,mincand,minr+1,minc+1);
+				break;
+		}
+		estr[511] = '\0';
+		fprint_explain(estr,*sudoku,verbosity);
+		sudoku->rating += 4 * minfcl;
+	}
+	
+	return(minrv);
+}
+
+/*
+ ******************************************************************************/
+/*
+int forcing_chain2(t_sudoku *sudoku, int verbosity, int *fcl) {
+	
 	int size = sudoku->size;
 	int r, c, cc, row, col, cand, c0, c1, elim[NUM_ET], ret, eofc0, eofc1;
 	int minr, minc, mincand, minfcl, minrv, minstartr, minstartc;
@@ -172,11 +360,11 @@ int forcing_chain(t_sudoku *sudoku, int verbosity, int *fcl) {
 	
 	return(minrv);
 }
-
+*/
 /*
  ******************************************************************************/
 /*
-int forcing_chain2(t_field grid[SIZE*SIZE][SIZE*SIZE], int size, int verbosity, int *fcl) {
+int forcing_chain1(t_field grid[SIZE*SIZE][SIZE*SIZE], int size, int verbosity, int *fcl) {
 	int r, c, cc, c0, c1, elim[NUM_ET], ret0, ret1, nnc, fcl0, fcl1;
 	t_field gridc0[SIZE*SIZE][SIZE*SIZE], gridc1[SIZE*SIZE][SIZE*SIZE];
 	t_field diffgridc0[SIZE*SIZE][SIZE*SIZE], diffgridc1[SIZE*SIZE][SIZE*SIZE];
