@@ -19,10 +19,10 @@ char progname[100];
 int action = 0;
 int max_rating = INT_MAX;
 int num_sudokus = 4;
-int verbosity = 0;
+int verbosity = 1;
 int nthread = 0;
-int symmgrid = 0;
-char *fn_out = NULL;
+t_symmetry symmgrid = symm_none;
+char fn_out[512] = "";
 FILE *inFile = NULL;
 
 pthread_mutex_t mut_input  = PTHREAD_MUTEX_INITIALIZER;
@@ -59,11 +59,11 @@ FILE *devurandom;
 int getoptions(int argc, char *argv[]) {
 	
 	int i, iet;
-	char *fn;
+	char *fn, sym;
 	
 	opterr = 0;
 	
-	while ((i = getopt (argc, argv, ":chNpVwsStf:i:n:o:r:v:P:")) != -1) {
+	while ((i = getopt (argc, argv, ":chNpstVwf:i:n:o:v:P:S:")) != -1) {
 		switch (i) {
 			case 'c':
 				action = 1;
@@ -81,7 +81,20 @@ int getoptions(int argc, char *argv[]) {
 				action = 5;
 				break;
 			case 'S':
-				symmgrid = 1;
+				if (sscanf(optarg,"%c",&sym) != 1) {
+					return(1);
+				}
+				switch(sym) {
+					case 'a':
+						symmgrid = symm_a;
+						break;
+					case 'p':
+						symmgrid = symm_p;
+						break;
+					default:
+						fprintf(stderr,"ERROR: unkown symmetry identifier after -S option. Try 'a' or 'p'.");
+						return(1);
+				}
 				break;
 			case 't':
 				nthread = sysconf(_SC_NPROCESSORS_ONLN);
@@ -112,13 +125,8 @@ int getoptions(int argc, char *argv[]) {
 				}
 				break;
 			case 'o':
-				fn_out = optarg;
-				break;
-			case 'r':
-/*				if ((sscanf(optarg,"%d",&max_rating) != 1) || (max_rating < 1)) {
-					return(1);
-				}*/
-				fprintf(stderr,"WARNING: ignoring option -r\n");
+				strncpy(fn_out,optarg,511);
+				fn_out[511] = '\0';
 				break;
 			case 'v':
 				if ((sscanf(optarg,"%d",&verbosity) != 1) || (verbosity < 0)) {
@@ -130,10 +138,10 @@ int getoptions(int argc, char *argv[]) {
 				exit(0);
 				break;
 			case 'P':
-				if(strlen(optarg) == (NUM_ET-2)) {
+				if(strlen(optarg) <= (NUM_ET-2)) {
 //					for (iet = 0; iet < NUM_ET; iet++) printf("%d ",dfcs[iet]);
 					for (iet = 1; iet < NUM_ET-1; iet++) dfcs[iet] = -1;
-					for (iet = 1; iet < NUM_ET-1; iet++) {
+					for (iet = 1; ( (iet < NUM_ET-1) && (optarg[iet-1] != '\0') ); iet++) {
 						switch(optarg[iet-1]) {
 							case '.':	dfcs[iet] = et_hs; break;
 							case '-':	dfcs[iet] = et_s;  break;
@@ -155,9 +163,13 @@ int getoptions(int argc, char *argv[]) {
 						}
 					}
 //					for (iet = 0; iet < NUM_ET; iet++) printf("%d ",dfcs[iet]);
-					for (iet = 1; iet < NUM_ET-2; iet++)
-						if (dfcs[iet] == -1)
-							return(1);
+//					for (iet = 1; iet < NUM_ET-2; iet++)
+//						if (dfcs[iet] == -1)
+//							return(1);
+					if ( (dfcs[1] != et_hs) && (dfcs[1] != et_s) ) {
+						fprintf (stderr,"%s: Error: First solution technique must be `.' or `-'\n",progname);
+						abort();
+					}
 				} else {
 					return(1);
 				}
@@ -188,7 +200,7 @@ int getoptions(int argc, char *argv[]) {
 void print_version(FILE *f) {
 	fprintf(f,"%s version %s\n",progname,VERSION);
 	fprintf(f,"\
-Copyright (c) 2007 Jens Baaran (http://baaran.de)\n\n\
+Copyright (c) 2007-2009 Jens Baaran (http://baaran.de)\n\n\
 This is free software. You may redistribute copies of this software\n\
 under the terms of the GNU General Public License Version 2, June 1991\n\
 (http://www.gnu.org/licenses/gpl.html). There is NO WARRANTY, to the\n\
@@ -216,8 +228,8 @@ Various elimination techniques for solving sudokus are implemented, starting\n\
 with easy techniques such as singles and hidden singles. Complex techniques\n\
 for candidate elimination (tuple / hidden tuple, line-block interaction,\n\
 forcing chain, X-wing, swordfish, jellyfish) are applied only if necessary,\n\
-reverting to backtracking at last resort. Solutions are checked for uniqueness.\n\
-\n\
+reverting to backtracking (guessing) at last resort. Solutions are checked for\n\
+uniqueness.\n\n\
 %s can display the solution process. When you set verbosity to 1, %s\n\
 will display a string of characters, each one stands for one solution step:\n\
  .\t hidden single\n\
@@ -250,17 +262,31 @@ tried, before reverting to guessing and starting a backtracking process.\n\
 \n\
 You can influence the order, in which the techniques above are applied, by\n\
 means of the -P option. As described above, the default order is\n\
-\".-dDbBtTqQWYXSJF\". If you want the steps to be applied in a different order,\n\
-just shuffle these characters and pass them to %s with the -P flag. Be sure to\n\
-specify all of the 16 characters exactly once, otherwise %s will abort.\n\
-For performance reasons not all techniques are applied during backtracking and\n\
-while searching for forcing chains, only the single and hidden single\n\
-techniques are applied. Forcing chains are only tried for cells with two\n\
-candidates. All of such cells are checked and %s will apply the shortest\n\
-forcing chain it finds (which is, by the way, not in all cases the shortest\n\
-forcing chain there is).\n\n\
-",progname,progname,progname,progname,progname);
+\".-dDbBtTqQWYXSJF\". If you want the steps to be applied in a different\n\
+order, just shuffle these characters and pass them to %s with the\n\
+ -P flag. Forcing chains are not applied during backtracking (i.e. after the\n\
+first value had to be guessed). While searching for forcing chains, only the\n\
+single and hidden single techniques are applied. Forcing chains are only tried\n\
+for cells with two candidates. All of such cells are checked and %s\n\
+will apply the shortest forcing chain it finds (which is, by the way, not in\n\
+all cases the shortest forcing chain there is). The sudoku solution and\n\
+generation performance can be improved greatly, if you leave out the 'q', 'Q'\n\
+and 'J' techniques, which occur only very rarely, but need a lot of\n\
+computation time.\n\n\
+",progname,progname,progname,progname);
 
+	fprintf(f,"\
+Sudoku generation:\n\t\n\
+All sudokus generated by %s are minimal sudokus, meaning that all givens\n\
+are required to ensure a unique solution to the sudoku. Optionally symmetric\n\
+sudokus can be generated by specifying '-S' on the command line. Generation\n\
+of symmetric sudokus requires much more time than the generation of\n\
+unsymmetric sudokus. It is not possible to specify a desired difficulty\n\
+rating for generation of sudokus. To produce sudokus with a certain\n\
+difficulty, just create a whole batch (say 100) and pick the ones with the\n\
+desired difficulty.\n\n\
+",progname);
+	
 	fprintf(f,"\
 Sudoku rating:\n\t\n\
 At first the sudoku is assigned a basic rating of 1, 10 or 100, depending on\n\
@@ -270,9 +296,9 @@ the most difficult technique required for its solution:\n\n\
  a basic rating of 100 is assigned for the techniques WYXSJFfeg.\n\n\
 Next, each solution step is assigned a difficulty rating, starting with 1 for\n\
 hidden singles up to %d for guesses during the backtracking process. In each\n\
-step the three easiest applicable techniques are taken into account. The\n\
+step the two easiest applicable techniques are taken into account. The\n\
 individual ratings for each solution step are added to the basic rating.\n\
-finally, 1 is subtracted from the sudoku rating for each cell to be solved,\n\
+Finally, 1 is subtracted from the sudoku rating for each cell to be solved,\n\
 so the smallest possible rating of 1 is assigned to all sudokus completely\n\
 solvable with the hidden singles technique.\n\n\
 ",NUM_ET-1);
@@ -284,7 +310,7 @@ solvable with the hidden singles technique.\n\n\
  ******************************************************************************/
 void print_usage(FILE *f) {
 	fprintf(f,"\
-Usage: %s <command> [options]\n\
+Usage: %s < command >  [ options ]\n\
 %s can solve and create sudokus. It can print 4 sudokus to a pdf or an svg\n\
 page and it comes with example cgi scripts for integration into www sites.\n\n\
 ",progname,progname);
@@ -293,7 +319,7 @@ page and it comes with example cgi scripts for integration into www sites.\n\n\
 Examples:\n\
  %s -s -v 1 < sud.in > sol.out  \tsolve sudokus\n\
  %s -c -n 20 > sud.out     \tcreate 20 sudokus\n\
- %s -p -f sud.pdf < sud.in \tprint 4 sudokus to sud.pdf & sud.pdf.svg\n\n\
+ %s -p -o sud.pdf < sud.in \tprint 4 sudokus to sud.pdf & sud.pdf.svg\n\n\
 ",progname,progname,progname);
 
 	fprintf(f,"\
@@ -318,13 +344,14 @@ Options (not all options work for every command):\n\
  \t when -P is not present. For further explanation of the sol_techn\n\
  \t string use the -h command (or see \"Sudoku solving\" below, if you are\n\
  \t looking at the documentation).\n\
- -v verbosity\t for explaining solution techniques (set it to 0, 1, 10 or 11)\n\
+ -v verbosity\t for explaining solution techniques (one of 0, 1, 10 or 11)\n\
  -f wsudognu\t location of wsudognu.cgi\n\
  \t wsudognu.cgi is a script necessary for processing the sudoku urls\n\
  \t created for the -s command with a verbosity of 11 (default value for\n\
  \t wsudognu is ./wsudognu.cgi)\n\
  -t\t use one thread for each available CPU (no pdf/svg file will be created)\n\
- -S\t create symmetrical sudokus (slow)\n\n\
+ -S symmetry\t create axi- or point-symmetrical sudokus\n\
+ \t slow, symmetry is 'a' or 'p'.\n\n\
 ");
 }
 
