@@ -245,12 +245,26 @@ int get_num_cell_with_stat1(t_sudoku sudoku, status stat) {
 	return(nstat);
 }
 
-/*
+/* check sudoku symmetry.
+ * 
+ * symmetry requirement is stored in symmgrid.
+ * 
+ * the row and column of the 1st cell that break symmetry is returned.
+ * 
+ * if point symmetry is required, vr and vc will be -1, if the sudoku is
+ * already symmetric.
+ * 
+ * if axisymmetry is required, vr and vc are -1 only,
+ * if there are two symmetry axes (horizontal and vertical).
  ******************************************************************************/
-int check_sudoku_symmetry(t_sudoku sudoku) {
+t_symmetry check_sudoku_symmetry(t_sudoku sudoku, int *vr, int *vc) {
 	
 	int size = sudoku.size;
-	int r, c, hsym=1, vsym=1;
+	int r, c, hsym=0, vsym=0, psym=0;
+	extern t_symmetry symmgrid;
+	
+	// initialize symmetry breaking row and column index
+	*vr = *vc = -1;
 
 	// make if conditions easier to program
 	for (r=0; r<size; r++) {
@@ -264,18 +278,56 @@ int check_sudoku_symmetry(t_sudoku sudoku) {
 	}
 	
 	// check for symmetry axes
-	for (r=0; r<size/2; r++) {
-		for (c=0; c<size; c++) {
-			if ((vsym == 1) && (sudoku.grid[r][c].stat != sudoku.grid[size-1-r][c].stat)) vsym = 0;
-			if ((hsym == 1) && (sudoku.grid[c][r].stat != sudoku.grid[c][size-1-r].stat)) hsym = 0;
+	if (symmgrid == symm_a) {
+		hsym = vsym = 1;
+		for (r=0; r<size/2+1; r++) {
+			for (c=0; c<size; c++) {
+				if ((vsym == 1) && (sudoku.grid[r][c].stat != sudoku.grid[size-1-r][c].stat)) {
+					vsym = 0;
+					if (sudoku.grid[r][c].stat == stat_free) {
+						*vr = r;
+						*vc = c;
+					} else {
+						*vr = size-1-r;
+						*vc = c;
+					}
+				}
+				if ((hsym == 1) && (sudoku.grid[c][r].stat != sudoku.grid[c][size-1-r].stat)) {
+					hsym = 0;
+					if (sudoku.grid[c][r].stat == stat_free) {
+						*vr = c;
+						*vc = r;
+					} else {
+						*vr = c;
+						*vc = size-1-r;
+					}
+				}
+			}
+		}
+	} else if (symmgrid == symm_p) {
+		psym = 1;
+		for (r=0; r<size/2+1; r++) {
+			for (c=0; c<size; c++) {
+				if ((psym == 1) && (sudoku.grid[r][c].stat != sudoku.grid[size-1-r][size-1-c].stat)) {
+					psym = 0;
+					if (sudoku.grid[r][c].stat == stat_free) {
+						*vr = r;
+						*vc = c;
+					} else {
+						*vr = size-1-r;
+						*vc = size-1-c;
+					}
+				}
+			}
 		}
 	}
 
 	// return
-	if      ((hsym == 1) && (vsym == 1)) return(3);
-	else if ((hsym == 1) && (vsym == 0)) return(2);
-	else if ((hsym == 0) && (vsym == 1)) return(1);
-	return(0);
+	if      ((hsym == 1) && (vsym == 1)) return(symm_hv);  // horiz & vert symmetry axis
+	else if (psym == 1) return(symm_p);  // point symmetry
+	else if (hsym == 1) return(symm_h);  // horizontal symmetry axis
+	else if (vsym == 1) return(symm_v);  // vertical symmetry axis
+	return(symm_none);
 }
 
 /* use random number from rand()
@@ -313,7 +365,42 @@ int get_random_cell(t_field grid[SIZE*SIZE][SIZE*SIZE], int size, status stat) {
 
 /* use random number from /dev/urandom
  ******************************************************************************/
-int get_random_cell2(t_field grid[SIZE*SIZE][SIZE*SIZE], int size, status stat) {
+int get_random_cell2(t_sudoku sudoku, status stat) {
+	int r, c, size = sudoku.size, nstat=0;
+	unsigned int istat;
+	extern FILE *devurandom;
+	
+	// count cells with status stat
+	nstat = get_num_cell_with_stat1(sudoku,stat);
+		
+	// pick random cell
+	if (devurandom != NULL) {
+		if (fread(&istat,sizeof(unsigned int),1,devurandom) != 1) {
+			fprintf(stderr,"error while trying to read from /dev/urandom\n");
+		}
+	} else {
+		fprintf(stderr,"error while trying to open /dev/urandom\n");
+	}
+
+	// if reading from /dev/urandom fails, just use uninitialized istat
+	istat = istat % nstat;
+
+	// find row / column
+	for (r=0; r<size; r++) {
+		for (c=0; c<size; c++) {
+			if ((stat == stat_undef) || (sudoku.grid[r][c].stat == stat)) {
+				if (istat > 0) istat--;
+				else return(r*size+c);
+			}
+		}
+	}
+	// return error
+	return(-1);
+}
+
+/* use random number from /dev/urandom
+ ******************************************************************************/
+int get_random_cell3(t_field grid[SIZE*SIZE][SIZE*SIZE], int size, status stat) {
 	int r, c, nstat=0;
 	unsigned int istat;
 	extern FILE *devurandom;
