@@ -19,12 +19,16 @@ extern pthread_mutex_t mut_stdout;
 int execute_thread_solve(void *arg[]) {
 	
 	FILE *in       = (FILE *) arg[0];
-//	int *verbosity = (int *) arg[1];
+	int *verbosity = (int *) arg[1];
 
 	// set verbosity to 0 for threads, because mutex protection for output after
 	// every solution step doesn't seem feasible. maybe each thread needs its
 	// own output file.
-	return(solve_sudoku_and_check_uniqueness(in,0));
+	if (*verbosity > 0) {
+		return(solve_sudoku_and_check_uniqueness(in,1));
+	} else {
+		return(solve_sudoku_and_check_uniqueness(in,0));
+	}
 }
 
 /*
@@ -55,17 +59,25 @@ int solve_sudoku_and_check_uniqueness(FILE *in, int verbosity) {
 		if (ret == 0) {
 			if (check_uniqueness1(sudoku,dsudoku) == 0) {
 				pthread_mutex_lock(&mut_stdout);
-				if (verbosity == 1) {
-					printf("\t%d\t%d\t%s\n",get_num_cell_with_stat1(sudoku,stat_ggiven) + get_num_cell_with_stat1(sudoku,stat_given),(int)dsudoku.rating,VERSION);
+				switch (verbosity) {
+					case 1:
+						fprint_sudoku(stdout, dsudoku);
+						printf("\n");
+//						fflush(stdout);
+						break;
+					case 2:
+						printf("\t%d\t%d\t%s\n",get_num_cell_with_stat1(sudoku,stat_ggiven) + get_num_cell_with_stat1(sudoku,stat_given),(int)dsudoku.rating,VERSION);
+						break;
 				}
 				pthread_mutex_unlock(&mut_stdout);
 			} else {
-				fflush(stdout);
+//				fflush(stdout);
 				fprintf(stderr,"\n\nSudoku has more than one solution\n");
 				if (DEBUG) fprint_sudoku(stderr,sudoku);
+				exit(4);
 			}
 		} else if (ret != 0) {
-			fflush(stdout);
+//			fflush(stdout);
 			fprintf(stderr,"\n\nNo solution found for sudoku\n");
 			fprint_sudoku(stderr,sudoku);
 			exit(3);
@@ -88,12 +100,12 @@ int solve_sudoku(t_sudoku *sudoku, int elim[NUM_ET], int depth, int verbosity, i
 	copy_sudoku(*sudoku,&origSudoku);
 	if (verbosity > 9) printf("\n");
 	ret = solve_sudoku_rec(sudoku,&tmpSudoku,elim,0,0,0,depth,verbosity, doRate);
-	if (verbosity == 1) {
+	if (verbosity == 2) {
 		printf("\t");
 		fprint_sudoku(stdout,origSudoku);
 	}
 	if (ret == 0) {
-		if (verbosity == 1) {
+		if (verbosity == 2) {
 			canoni(origSudoku,&tmpSudoku);
 			printf("\t");
 			fprint_sudoku(stdout,tmpSudoku);
@@ -156,10 +168,18 @@ int solve_sudoku_rec(t_sudoku *sudoku, t_sudoku *dsudoku,
 	t_sudoku esudoku;
 	int ret = 1, r1=-1, c1=-1, cc1=-1;
 	int size = sudoku->size;
+	int backtrack_depth;
+	extern int dfcs[];
 	
 	// forcing chains don't work during backtracking
-	elim_type backtrack_depth = depth < (et_fc - 1) ? depth : (et_fc-1);
-	
+	backtrack_depth = 1;
+	while ( (dfcs[backtrack_depth] != et_fc)
+			&& (backtrack_depth <= depth)
+			&& (backtrack_depth < (NUM_ET-1)) ) {
+		backtrack_depth++;
+	}
+	backtrack_depth--;
+
 	// dgrid is temp grid, if there have been guesses it contains latest guess
 	// grid is original grid before last guess
 	
@@ -180,7 +200,7 @@ int solve_sudoku_rec(t_sudoku *sudoku, t_sudoku *dsudoku,
 			}
 
 			// guess
-			if (verbosity == 1) printf("g");
+			if (verbosity == 2) printf("g");
 			if (verbosity > 9)  printf("g  r%dc%d (%d) guess\n",r1+1,c1+1,cc1+1);
 			// for every new guess make another copy of the grid
 			dsudoku->grid[r1][c1].stat  = stat_guess;
@@ -206,7 +226,7 @@ int solve_sudoku_rec(t_sudoku *sudoku, t_sudoku *dsudoku,
 			
 			if (sudoku->grid[r][c].stat == stat_guess) {
 				// wrong guess, remove candidate from original grid
-				if (verbosity == 1) printf("w");
+				if (verbosity == 2) printf("w");
 				if (verbosity > 9)  printf("w  r%dc%d wrong guess, remove candidate %d\n",r+1,c+1,cc+1);
 				sudoku->grid[r][c].cand[cc] = 0;
 				sudoku->grid[r][c].ncand -= 1;
@@ -228,15 +248,17 @@ int solve_sudoku_rec(t_sudoku *sudoku, t_sudoku *dsudoku,
  ******************************************************************************/
 int eliminate(t_sudoku *sudoku, int elim[NUM_ET], int depth, int verbosity, int doRate) {
 
-	int i, j, e, ne = 3; // averaged difficulties of the (ne) easiest techniques determine difficulty of a solution step
+	int i, j, e, ne = 2; // averaged difficulties of the (ne) easiest techniques determine difficulty of a solution step
 	char str[1024] = "";
 	extern int dfcs[];
 	
 	do {
 		i = e = 0;
 		if (doRate) {
-			for (j = et_hs; ((j <= et_jf) && (i < ne)); j++) {
-				switch(j) {
+//			for (j = et_hs; ((j <= et_jf) && (i < ne)); j++) {
+			//				switch(j) {
+			for (j=1; ( (j < NUM_ET-1) && (i < ne)); j++) {
+				switch(dfcs[j]) {
 				case et_hs:
 					e = hidden_single(sudoku,0,0);
 					if ((verbosity > 2) && (verbosity < 11)) sprintf(str+strlen(str),".%d,",e);
@@ -297,15 +319,21 @@ int eliminate(t_sudoku *sudoku, int elim[NUM_ET], int depth, int verbosity, int
 					e = fish(sudoku,4,0,0);
 					if ((verbosity > 2) && (verbosity < 11)) sprintf(str+strlen(str),"J%d,",e);
 					break;
+				case et_fc:
+					e = 0;
+					break;
+				case -1:
+					e = 0;
+					break;
 				}
 				e = ( (e+i) > ne ? (ne-i) : e );
 				i += e;
-				sudoku->rating += (e * dfcs[j]) / ((float) ne);
+				sudoku->rating += (e * j) / ((float) ne);
 			}
 			if ((i < ne) && (get_num_cell_with_stat1(*sudoku,stat_free) > 0)) {
 				sudoku->rating += ((ne - i) * (et_jf + 1)) / ((float) ne);
 			}
-			if ((verbosity > 2) && (verbosity < 11)) {
+			if ((verbosity > 2) && (verbosity < 11) && (strlen(str) > 0) ) {
 				sprintf(str+strlen(str)-1,")");
 				fprintf(stderr," rating %7.1f\t\t(%s\n",sudoku->rating,str);
 				strcpy(str,"");
@@ -329,13 +357,14 @@ int apply_solution_technique1(t_sudoku *sudoku, int elim[NUM_ET], int depth, int
 	// techniques as integers from 1 (et_hs, hidden single) through et_fc (forcing
 	// chain), as defined in header file sudoku.h.
 	// dfcs[0] is wrong guess, dfcs[NUM_ET-1] is guess
+	// dfcs[i] = -1 means: do nothing.
 	for (iet=1; iet<NUM_ET-1; iet++) {
 		if (depth >= iet) {
 			switch(dfcs[iet]) {
 				case et_hs:
 					if (hidden_single(sudoku,1,verbosity) > 0) {
 						new = 1;
-						if (verbosity == 1) fprintf(out,".");
+						if (verbosity == 2) fprintf(out,".");
 						elim[dfcs[iet]] += 1;
 					}
 					break;
@@ -343,85 +372,85 @@ int apply_solution_technique1(t_sudoku *sudoku, int elim[NUM_ET], int depth, int
 					if (single(sudoku,1,verbosity) > 0) {
 						new = 1;
 						elim[dfcs[iet]] += 1;
-						if (verbosity == 1) fprintf(out,"-");
+						if (verbosity == 2) fprintf(out,"-");
 					}
 					break;
 				case et_hd:
 					if ((new = hidden_tuple(sudoku,2,1,verbosity)) > 0) {
 						elim[et_hd] += 1;
-						if (verbosity == 1) fprintf(out,"d");
+						if (verbosity == 2) fprintf(out,"d");
 					}
 					break;
 				case et_d:
 					if ((new = tuple(sudoku,2,1,verbosity)) > 0) {
 						elim[et_d] += 1;
-						if (verbosity == 1) fprintf(out,"D");
+						if (verbosity == 2) fprintf(out,"D");
 					}
 					break;
 				case et_b:
 					if ((new = block_line(sudoku,1,verbosity)) > 0) {
 						elim[dfcs[iet]] += 1;
-						if (verbosity == 1) fprintf(out,"b");
+						if (verbosity == 2) fprintf(out,"b");
 					}
 					break;
 				case et_l:
 					if ((new = line_block(sudoku,1,verbosity)) > 0) {
 						elim[dfcs[iet]] += 1;
-						if (verbosity == 1) fprintf(out,"B");
+						if (verbosity == 2) fprintf(out,"B");
 					}
 					break;
 				case et_ht:
 					if ((new = hidden_tuple(sudoku,3,1,verbosity)) > 0) {
 						elim[et_ht] += 1;
-						if (verbosity == 1) fprintf(out,"t");
+						if (verbosity == 2) fprintf(out,"t");
 					}
 					break;
 				case et_t:
 					if ((new = tuple(sudoku,3,1,verbosity)) > 0) {
 						elim[et_t] += 1;
-						if (verbosity == 1) fprintf(out,"T");
+						if (verbosity == 2) fprintf(out,"T");
 					}
 					break;
 				case et_hq:
 					if ((new = hidden_tuple(sudoku,4,1,verbosity)) > 0) {
 						elim[et_hq] += 1;
-						if (verbosity == 1) fprintf(out,"q");
+						if (verbosity == 2) fprintf(out,"q");
 					}
 					break;
 				case et_q:
 					if ((new = tuple(sudoku,4,1,verbosity)) > 0) {
 						elim[et_q] += 1;
-						if (verbosity == 1) fprintf(out,"Q");
+						if (verbosity == 2) fprintf(out,"Q");
 					}
 					break;
 				case et_xy:
 					if ((new = xy_wing(sudoku,1,verbosity)) > 0) {
 						elim[et_xy] += 1;
-						if (verbosity == 1) fprintf(out,"W");
+						if (verbosity == 2) fprintf(out,"W");
 					}
 					break;
 				case et_xyz:
 					if ((new = xyz_wing(sudoku,1,verbosity)) > 0) {
 						elim[et_xyz] += 1;
-						if (verbosity == 1) fprintf(out,"Y");
+						if (verbosity == 2) fprintf(out,"Y");
 					}
 					break;
 				case et_xw:
 					if ((new = fish(sudoku,2,1,verbosity)) > 0) {
 						elim[et_xw] += 1;
-						if (verbosity == 1) fprintf(out,"X");
+						if (verbosity == 2) fprintf(out,"X");
 					}
 					break;
 				case et_sf:
 					if ((new = fish(sudoku,3,1,verbosity)) > 0) {
 						elim[et_sf] += 1;
-						if (verbosity == 1) fprintf(out,"S");
+						if (verbosity == 2) fprintf(out,"S");
 					}
 					break;
 				case et_jf:
 					if ((new = fish(sudoku,4,1,verbosity)) > 0) {
 						elim[et_jf] += 1;
-						if (verbosity == 1) fprintf(out,"J");
+						if (verbosity == 2) fprintf(out,"J");
 					}
 					break;
 				case et_fc:
@@ -429,19 +458,21 @@ int apply_solution_technique1(t_sudoku *sudoku, int elim[NUM_ET], int depth, int
 						switch(new) {
 							case 1:
 								elim[et_g] += 1;
-								if (verbosity == 1) fprintf(out,"e%d",fcl);
+								if (verbosity == 2) fprintf(out,"e%d",fcl);
 								break;
 							case 2:
 								elim[et_fc] += 1;
-								if (verbosity == 1) fprintf(out,"f%d",fcl);
+								if (verbosity == 2) fprintf(out,"f%d",fcl);
 								break;
 							case 3:
 								elim[et_fc] += 1;
-								if (verbosity == 1) fprintf(out,"F%d",fcl);
+								if (verbosity == 2) fprintf(out,"F%d",fcl);
 								break;
 						}
 					}
 					break;
+				case -1:
+					break;
 				default:
 					fprintf(out,"WARNING: unknown elim type: %d\n",dfcs[iet]);
 					break;
