@@ -15,7 +15,7 @@ extern int action;
 extern int max_rating;
 extern int verbosity;
 extern int nthread;
-extern char *fn_out;
+extern char fn_out[512];
 extern char progname[100];
 extern FILE *explFile;
 extern FILE *devurandom;
@@ -27,7 +27,7 @@ int main (int argc, char *argv[]) {
 	
 	clock_t t0;
 	float ct1;
-	int i, elim[NUM_ET], ret;
+	int i, elim[NUM_ET], ret, d0, d1;
 	t_sudoku sudoku, foursudokus[4];
 	extern int num_sudokus;
 
@@ -60,14 +60,14 @@ int main (int argc, char *argv[]) {
 	// --------------------------
 	if (action == 1) {
 		devurandom = fopen("/dev/urandom","rb");
+		fprintf(stderr,"\nCreating %d sudokus\n",num_sudokus);
 
 		if (nthread < 1) {
 			
 			for (i=0; i<num_sudokus; i++) {
-				fprintf(stderr,"\nCreating sudoku %d\n",i+1);
 				reset_sudoku(&(foursudokus[i%4]));
-				foursudokus[i%4].rating = create_sudoku(&(foursudokus[i%4]));
-				fprintf(stderr,"\n");
+				foursudokus[i%4].rating = create_sudoku(&(foursudokus[i%4]),verbosity);
+//				fprintf(stderr,"\n");
 			}
 #ifdef HARULIB
 			if (num_sudokus == 4) sudokus_to_pdf(fn_out,foursudokus);
@@ -75,14 +75,18 @@ int main (int argc, char *argv[]) {
 			sudokus_to_svg(fn_out,foursudokus);
 		} else {
 
-			fprintf(stderr,"\nCreating sudoku\n");
-	
 			// allocate
 			thread = (pthread_t*) calloc(nthread,sizeof(pthread_t));
-
+			targv  = (void***)   calloc(nthread,sizeof(void**));
+			for (ithread=0; ithread<nthread; ithread++) {
+				targv[ithread] = (void**) calloc(1,sizeof(void*));
+				targv[ithread][0] = &verbosity;
+			}
+			attr_p = NULL;
+			
 			// start threads
 			for (ithread=0; ithread<nthread; ithread++) {
-				if ((rv = pthread_create(&(thread[ithread]), attr_p, (void *) execute_thread_create, (void *) NULL)) != 0) {
+				if ((rv = pthread_create(&(thread[ithread]), attr_p, (void *) execute_thread_create, (void *) targv[ithread])) != 0) {
 					fprintf(stderr,"Error %d!\n%s\n",rv,strerror(rv));
 				}
 			}
@@ -93,11 +97,13 @@ int main (int argc, char *argv[]) {
 				if ((rv = pthread_join(thread[ithread], (void **) &trv)) != 0) {
 					fprintf(stderr,"Error %d!\n%s\n",rv,strerror(rv));
 				}
-				fprintf(stderr,"Thread %d produced %d sudokus.\n",ithread,trv);
-				nsud += trv;
 			}
 
 			// message
+			for (ithread=0; ithread<nthread; ithread++) {
+				fprintf(stderr,"Thread %d produced %d sudokus.\n",ithread,trv);
+				nsud += trv;
+			}
 			fprintf(stderr,"created %d sudokus in %d threads.\n",nsud,nthread);
 		
 			// free
@@ -135,6 +141,10 @@ int main (int argc, char *argv[]) {
 				if (check_uniqueness1(foursudokus[i],sudoku) == 0) {
 					foursudokus[i].rating = sudoku.rating;
 					printf("\nFound unique solution\nsolution time: %d msec\n",(int)(ct1*1000));
+					// if sudoku is symmetric, beautify it
+					if (check_sudoku_symmetry(foursudokus[i], &d0, &d1) != symm_none) {
+						beautify_symmetric_sudoku(&(foursudokus[i]));
+					}
 				} else {
 					printf("\nThis sudoku has more than one solution, resetting grid.\n");
 					reset_sudoku(&(foursudokus[i]));
